/*
 * Written by Cordell Zebrose & Michael Metzler
 * Node.cpp
 *
 * Node is a GameObject which contains all the information for
 *	a single node. Each node is connected to up to 4 Roads, 
 *	up to 2 buildings, and up to 9 troops of a each type.
 */

#include "Node.h"
#include "Road.h"
#include "WorldManager.h"
#include "LogManager.h"
#include "ResourceManager.h"
#include "GraphicsManager.h"

#include <cstdlib>
#include <sstream>
#include <string>

//////////////////////////////////////////////////////////////////////////////////////

// arguments: 	x - the x position on the node graph, must be converted to world coordinates
//		y - the y position on the node graph, must be converted to world coordinates
// creates an empty unowned node which isn't connected to any roads and has no units
Node::Node(int x, int y) {
	//set solid
	setSolid(false);

	//set position
	WorldManager &wm = WorldManager::getInstance();
	wm.moveObj(this, Position(x*14+1, y*8+7));
	LogManager &lm = LogManager::getInstance();
	lm.writeLog("Node::Node: Created a node at world position %d,%d", x*13, y*8);

	//set Sprite
	LogManager &log_manager = LogManager::getInstance();
	ResourceManager &rm = ResourceManager::getInstance();
	Sprite *p_temp_sprite = rm.getSprite("node");
	if (!p_temp_sprite) {
		log_manager.writeLog("Node::Node(): Warning! Sprite '%s' not found", 
				 "node");
		return;
	}
	p_temp_sprite->setColor(NOT_OWNED_COLOR);
	setSprite(p_temp_sprite); 
	setSpriteCentered(false);

	//set Color
	color = NOT_OWNED_COLOR;

	//set Type
	setType("Node");

	//set roads
	p_road[NORTH] = NULL;
	p_road[SOUTH] = NULL;
	p_road[EAST] = NULL;
	p_road[WEST] = NULL;

	road_movements[NORTH] = Node::NONE;
	road_movements[SOUTH] = Node::NONE;
	road_movements[EAST] = Node::NONE;
	road_movements[WEST] = Node::NONE;

	//set garrison
	garrison[SWORDSMEN] = 0;
	garrison[HORSEMEN] = 0;
	garrison[PIKEMEN] = 0;

	//set buildings
	buildings[0] = NO_BUILDING;
	buildings[1] = NO_BUILDING;
	
	//set costs
	cost[NO_BUILDING] = COST_NO_BUILDING;
	cost[SWORDSMEN_BARRACKS] = COST_S_BUILDING;
	cost[HORSEMEN_BARRACKS] = COST_H_BUILDING;
	cost[PIKEMEN_BARRACKS] = COST_P_BUILDING;
	cost[MINE] = COST_M_BUILDING;
	
	//set surplus
	surplus_resources = 0;
	
	//set owner
	owner = NOT_OWNED;

	//set is selected
	is_selected = false;
}//Node

Node::~Node() {
	
}//~Node
		
////////////////////////////////////////////////////////////////////////////////////////
		
int Node::eventHandler(Event *e) {
	if(e->getType() == STEP_EVENT)
	{
		step();
		return 1;
	}	

	return 0;
}//eventHandler


/////////////////////////////////////////////////////////////////////////////////////////

int Node::getSword() {
	return garrison[SWORDSMEN];
}//getSword

int Node::getHorse() {
	return garrison[HORSEMEN];
}//getHorse

int Node::getPike() {
	return garrison[PIKEMEN];
}//getPike
  
/////////////////////////////////////////////////////////////////////////////////////////
  
void Node::setRoad(Road *new_road, Direction direction) {
	p_road[direction] = new_road;
}//setRoad

Road *Node::getRoad(Direction direction) {
	return p_road[direction];
}//getRoad
		
////////////////////////////////////////////////////////////////////////////////////////
		
// argument: unit_type - the type of unit being added
// adds a single unit to the node
// return: 0 if unit doesn't return, 1 if unit returns, and -1 if error
int Node::addUnit(Unit unit_type, int new_unit_owner) {
	if(garrison[unit_type] >= MAX_GARRISON)
	{
		return 1;
	}else
	{
		if(new_unit_owner == owner)
		{
			#ifdef DEBUG3
				LogManager &lm = LogManager::getInstance();
				lm.writeLog("Node::addUnit: Added unit to friendly Node #%d", getId());
			#endif
			garrison[unit_type]++;
			return 0;
		}

		Unit garrison_type = Node::SWORDSMEN;
		int garrison_type_iter = 0;
		while(garrison_type_iter < 2 && garrison[garrison_type] == 0)
		{
			garrison_type_iter++;
			garrison_type = static_cast<Unit>(garrison_type_iter);
		}

		if(garrison[garrison_type] == 0)
		{
			#ifdef DEBUG3
				LogManager &lm = LogManager::getInstance();
				lm.writeLog("Node::addUnit: Conquered Node #%d for Owner %d",
						getId(), new_unit_owner);
			#endif
			garrison[unit_type]++;
			setOwner(new_unit_owner);
			return 0;
		}else
		{
			#ifdef DEBUG3
				LogManager &lm = LogManager::getInstance();
			#endif
			if(unitCombat(unit_type, garrison_type))
			{
				#ifdef DEBUG3
					lm.writeLog("Node::addUnit: Attacking unit has killed a garrison");
				#endif
				garrison[garrison_type]--;
				return 1;
			}else
			{
				#ifdef DEBUG3
					lm.writeLog("Node::addUnit: Attacking unit has died");
				#endif
				return 0;
			}
		}
	}//if-else
}//addUnit

////////////////////////////////////////////////////////////////////////////////////////

// arguments: 	direction - the direction to move in
//		unit_type - the type of unit being added
// Starts to move all units of type unit_type in the 
// 	direction indicated until all units have moved.
// return: 0 if successful, 1 if there are no units
//		available, and -1 if the move failed
int Node::moveUnits(Unit unit_type, int direction) {
	if(garrison[unit_type] == 0 || p_road[direction] == NULL)
	{
		return 1;
	}else
	{
		road_movements[direction] = unit_type;
		return 0;
	}
}//moveUnits

/////////////////////////////////////////////////////////////////////////////////////////

// move units for one step
void Node::step() {
	for(int i = 0; i < 4; i++)
	{
		if(road_movements[i] != Node::NONE)
		{
			if(garrison[road_movements[i]] == 0)
			{
				road_movements[i] = Node::NONE;
			}else
			{
				int movement_direction;
				switch(i)
				{
					case Node::NORTH:
					case Node::EAST:
						movement_direction = NORTHEAST_MOVEMENT;
						break;
					case Node::SOUTH:
					case Node::WEST:
						movement_direction = SOUTHWEST_MOVEMENT;
						break;
					default:
						movement_direction = NO_MOVEMENT;
				};

				int movement_check = 
					p_road[i]->addUnit(road_movements[i], movement_direction, owner);
				switch(movement_check)
				{
					case 1:
						break;
					case 0:
						garrison[road_movements[i]]--;
						break;
					default:
						LogManager &lm = LogManager::getInstance();
						lm.writeLog("Node::step: Error on return from Road::addUnit");
						break;
				};
			}//if-else
		}//if
	}//for
}//step

////////////////////////////////////////////////////////////////////////////////////////

void Node::switchColor(int new_color) {
	color = new_color;
	Sprite *p_new_sprite = getSprite();
	p_new_sprite->setColor(new_color);
	setSprite(p_new_sprite);
}//switchColor

///////////////////////////////////////////////////////////////////////////////////////

void Node::setOwner(int new_owner) {
	#ifdef DEBUG3
		LogManager &lm = LogManager::getInstance();
		lm.writeLog("Node::setOwner: Setting owner of Node #%d to %d", getId(), new_owner);
	#endif

	owner = new_owner;
	Sprite *p_new_sprite = getSprite();
	switch(owner)
	{
	case NOT_OWNED:
		p_new_sprite->setColor(NOT_OWNED_COLOR);
		break;
	case PLAYER_OWNED:
		p_new_sprite->setColor(PLAYER_OWNED_COLOR);
		break;
	default:
		p_new_sprite->setColor(AI_OWNED_COLOR);
	};
	setIsSelected(is_selected);	
	//switchColor(p_new_sprite->getColor());
	setSprite(p_new_sprite);

	#ifdef DEBUG3
		lm.writeLog("Node::setOwner: Setting color of Node #%d to %d", getId(), 
				p_new_sprite->getColor());
	#endif
}//setOwner

int Node::getOwner() {
	return owner;
}//getOwner

///////////////////////////////////////////////////////////////////////////////////////

void Node::setIsSelected(bool new_is_selected) {
	is_selected = new_is_selected;
	Sprite *p_new_sprite = getSprite();
	switch(owner)
	{
	case NOT_OWNED:
		if(is_selected)
			p_new_sprite->setColor(NOT_OWNED_SELECTED);
		else
			p_new_sprite->setColor(NOT_OWNED_COLOR);
		break;
	case PLAYER_OWNED:
		if(is_selected)
			p_new_sprite->setColor(PLAYER_OWNED_SELECTED);
		else
			p_new_sprite->setColor(PLAYER_OWNED_COLOR);
		break;
	default:
		if(is_selected)
			p_new_sprite->setColor(AI_OWNED_SELECTED);
		else
			p_new_sprite->setColor(AI_OWNED_COLOR);
	};
	switchColor(p_new_sprite->getColor());
}//setIsSelected

bool Node::getIsSelected() {
	return is_selected;
}//getIsSelected

///////////////////////////////////////////////////////////////////////////////////////////

void Node::buildingStep() {
	for(int i = 0; i < 2; i++)
	{
		switch(buildings[i])
		{
			case SWORDSMEN_BARRACKS:
				if(addUnit(Node::SWORDSMEN, owner) == 0)
					surplus_resources -= COST_SWORDSMEN;
				break;
			case HORSEMEN_BARRACKS:
				if(addUnit(Node::HORSEMEN, owner) == 0)
					surplus_resources -= COST_HORSEMEN;
				break;
			case PIKEMEN_BARRACKS:
				if(addUnit(Node::PIKEMEN, owner) == 0)
					surplus_resources -= COST_PIKEMEN;
				break;
			case MINE:
				surplus_resources += MINE_OUTPUT;
				break;
		};
	}
}//buildingStep

int Node::getBuilding(int index) {
	if(index == 0 || index == 1)
		return buildings[index];
	return NO_BUILDING;
}//getBuilding

// returns the cost of the building
int Node::checkBuildingCost(int building_type) {
	return cost[building_type];
}//checkBuildingCost

// Adds a building to the node according to building_type
void Node::addBuilding(int building_type) {
	int index;
	if(buildings[0] == NO_BUILDING)
		index = 0;
	else if(buildings[1] == NO_BUILDING)
		index = 1;
	else
		return;
	buildings[index] = building_type;
}//addBuilding

// Removes all the buildings on the node
// returns the costs of those buildings
int Node::removeBuildings() {
	int total_loss = cost[buildings[0]] + cost[buildings[1]];
	buildings[0] = NO_BUILDING;
	buildings[1] = NO_BUILDING;
	return total_loss;
}//removeBuildings

//////////////////////////////////////////////////////////////////////////////////////////

int Node::getResources() {
	int to_ret = surplus_resources;
	surplus_resources = 0;
	return to_ret;
}//getResources

//////////////////////////////////////////////////////////////////////////////////////////

// override the GameObject draw method, so that the color
void Node::draw() {
	switchColor(color);

#ifdef DEBUG3
	LogManager &lm = LogManager::getInstance();
	lm.writeLog("Node::draw: Drawing Node #%d, Sprite Color is %d", getId(),
			getSprite()->getColor());
#endif
	GameObject::draw();

	GraphicsManager &gm = GraphicsManager::getInstance();
	Position obj_pos = getPos();
	int drawn_y = obj_pos.getY();
	int drawn_x = obj_pos.getX();
	
	// draw buildings
	drawn_x += 3;
	drawn_y += 1;
	for(int i = 0; i < 2; i++)
	{
		switch(buildings[i])
		{
			case SWORDSMEN_BARRACKS:
				gm.drawCh(Position(drawn_x, drawn_y+i), CHAR_S_BUILDING, getSprite()->getColor());
				break;
			case HORSEMEN_BARRACKS:
				gm.drawCh(Position(drawn_x, drawn_y+i), CHAR_H_BUILDING, getSprite()->getColor());
				break;
			case PIKEMEN_BARRACKS:
				gm.drawCh(Position(drawn_x, drawn_y+i), CHAR_P_BUILDING, getSprite()->getColor());
				break;
			case MINE:
				gm.drawCh(Position(drawn_x, drawn_y+i), CHAR_M_BUILDING, getSprite()->getColor());
				break;
			default:
				gm.drawCh(Position(drawn_x, drawn_y+i), CHAR_NO_BUILDING, getSprite()->getColor());
				break;
		};
	}
	
	// draw unit numbers
	drawn_y = obj_pos.getY();
	drawn_y += 3;

	for(int i = 0; i < 3; i++)
	{
		std::ostringstream sout;
		sout << garrison[i];
		std::string garrison_str = sout.str();
		drawn_x = obj_pos.getX() + 1 + i*2;
		gm.drawCh(Position(drawn_x, drawn_y), garrison_str[0], getSprite()->getColor());
	}
	
	// draw the unit movements
	for(int i = 0; i < 4; i++)
	{
		if(road_movements[i] != Node::NONE)
		{
			int gate_x = getPos().getX();
			int gate_y = getPos().getY();
			
			if(i == Node::NORTH || i == Node::SOUTH)
				gate_x += 3;
			if(i == Node::EAST || i == Node::WEST)
				gate_y += 2;
			if(i == Node::SOUTH)
				gate_y += 4;
			if(i == Node::EAST)
				gate_x += 6;
			
			gm.drawCh(Position(gate_x,gate_y), '+', NOT_OWNED_SELECTED);
		}
	}
}//draw

//////////////////////////////////////////////////////////////////////////////////////////

// fights combat between the given units and return the winner
// return true if unit_a wins, and false if unit_b wins
bool Node::unitCombat(Unit unit_a, Unit unit_b) {
		int combat_value = rand() % 10;
	#ifdef DEBUG3
		LogManager &lm = LogManager::getInstance();
		lm.writeLog("Node::unitCombat: combat value = %d", combat_value);
	#endif	
	if(unit_a == unit_b)
		return rand()%10 < COMBAT_FAIR;

	if(unit_b == Node::NONE)
		return true;

	if(unit_a == Node::NONE)
		return false;

	if(unit_a == unit_b + 1)
		return rand()%10 < COMBAT_UNFAIR;
	
	if(unit_b == unit_a + 1)
		return rand()%10 > COMBAT_UNFAIR;

	if(unit_b == Node::PIKEMEN && unit_a == Node::SWORDSMEN)
		return rand()%10 < COMBAT_UNFAIR;

	if(unit_a == Node::SWORDSMEN && unit_b == Node::PIKEMEN)
		return rand()%10 > COMBAT_UNFAIR;

	return rand()%10 < COMBAT_FAIR;
}//unitCombat
