/*
 * Written by Cordell Zebrose & Michael Metzler
 * Road.cpp
 *
 * Road is a GameObject which contains all the information for
 *	a single road section. Each road is either a vertical road
 *  or a horizontal road. A road section is connected to exactly
 *  2 nodes and contains up to 3 units.
 */

#include "Road.h"
#include "Node.h"

#include "WorldManager.h"
#include "LogManager.h"
#include "ResourceManager.h"
#include "GraphicsManager.h"
#include "StepEvent.h"

////////////////////////////////////////////////////////////////////////////////////////

// All roads must be connected to two nodes and be either vertical or horizontal
Road::Road(bool init_is_vertical, Node *init_northeast_node, Node *init_southwest_node) {
	//set orientation
	is_vertical = init_is_vertical;
	
	// set solid
	setSolid(false);

	// register for step event
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this, STEP_EVENT);

	//set Sprite
	LogManager &log_manager = LogManager::getInstance();
	ResourceManager &rm = ResourceManager::getInstance();
	Sprite *p_temp_sprite;	
	if(is_vertical)
		p_temp_sprite = rm.getSprite("road-vertical");
	else
		p_temp_sprite = rm.getSprite("road-horizontal");
	if (!p_temp_sprite) {
		log_manager.writeLog("Road::Road(): Warning! Sprite '%s' not found", 
				 "road");
		return;
	}
	p_temp_sprite->setColor(NOT_OWNED_COLOR);
	setSprite(p_temp_sprite); 
	setSpriteCentered(false);

	//set position
	initPosition(init_northeast_node);

	//set Type
	if(is_vertical)
		setType("Road-Vertical");
	else
		setType("Road-Horizontal");

	//set nodes
	if(is_vertical)
	{
		p_node[Node::NORTH] = init_northeast_node;
		p_node[Node::SOUTH] = init_southwest_node;

		p_node[Node::NORTH]->setRoad(this, Node::SOUTH);
		p_node[Node::SOUTH]->setRoad(this, Node::NORTH);
	}else
	{
		p_node[Node::EAST] = init_northeast_node;
		p_node[Node::WEST] = init_southwest_node;

		p_node[Node::EAST]->setRoad(this, Node::WEST);
		p_node[Node::WEST]->setRoad(this, Node::EAST);
	}

	//set units
	units[NORTHEAST_ROAD] = Node::NONE;
	units[MIDDLE_ROAD] = Node::NONE;
	units[SOUTHWEST_ROAD] = Node::NONE;
	
	//set unit movement
	unit_direction[NORTHEAST_ROAD] = NO_MOVEMENT;
	unit_direction[MIDDLE_ROAD] = NO_MOVEMENT;
	unit_direction[SOUTHWEST_ROAD] = NO_MOVEMENT;

	//set unit owner
	unit_owner[NORTHEAST_ROAD] = NOT_OWNED;
	unit_owner[MIDDLE_ROAD] = NOT_OWNED;
	unit_owner[SOUTHWEST_ROAD] = NOT_OWNED;

	//set move_slowdown & move_countdown
	move_slowdown = 40;
	move_countdown = move_slowdown;

#ifdef DEBUG3
	LogManager &lm = LogManager::getInstance();
			
	if(is_vertical)
		lm.writeLog("Road::Road: Created Vertical Road %d (N Node #%d-%d-%d-%d-S Node #%d)",
			getId(), p_node[Node::NORTH]->getId(), units[NORTHEAST_ROAD], units[MIDDLE_ROAD], 
			units[SOUTHWEST_ROAD], p_node[Node::SOUTH]->getId());
	else
		lm.writeLog("Road::Road: Created Horizontal Road %d (W Node #%d-%d-%d-%d-E Node #%d)",
			getId(), p_node[Node::WEST]->getId(), units[NORTHEAST_ROAD], units[MIDDLE_ROAD], 
			units[SOUTHWEST_ROAD], p_node[Node::EAST]->getId());
#endif	
}//Road

Road::~Road() {

}//~Road
	
///////////////////////////////////////////////////////////////////////////////////////

void Road::initPosition(Node *p_northeast_node) {
	Position node_pos = p_northeast_node->getPos();

	if(is_vertical)
	{
		node_pos.setX(node_pos.getX()+4);
		node_pos.setY(node_pos.getY()+5);
	}else
	{
		node_pos.setX(node_pos.getX()-6);
		node_pos.setY(node_pos.getY()+2);
	}

	WorldManager &wm = WorldManager::getInstance();
	wm.moveObj(this, node_pos);
}//initPosition

////////////////////////////////////////////////////////////////////////////////////////

// move all the units on the road, if a unit moves off the road then send it to the connected
//	node. NOTE: all three unit lists will be modified by this function
void Road::moveUnits() {
#ifdef DEBUG3
	LogManager &lm = LogManager::getInstance();
	lm.writeLog("Road::moveUnits: Moving units on Road %d (NE-%d-%d-%d-SW)",
			getId(), units[NORTHEAST_ROAD], units[MIDDLE_ROAD], units[SOUTHWEST_ROAD]);
	lm.writeLog("Road::moveUnits: Units movements (NE(-)=(%d)=(%d)=(%d)=SW(+))",
			unit_direction[NORTHEAST_ROAD], unit_direction[MIDDLE_ROAD], unit_direction[SOUTHWEST_ROAD]);
#endif	

	// check the northeast section for northeast movement
	if(unit_direction[NORTHEAST_ROAD] == NORTHEAST_MOVEMENT)
	{
		Node *p_northeast_node;
		if(is_vertical)
			p_northeast_node = p_node[Node::NORTH];
		else
			p_northeast_node = p_node[Node::EAST];

		if(!p_northeast_node->addUnit(units[NORTHEAST_ROAD], unit_owner[NORTHEAST_ROAD]))
		{
			units[NORTHEAST_ROAD] = Node::NONE;
			unit_owner[NORTHEAST_ROAD] = NOT_OWNED;
			unit_direction[NORTHEAST_ROAD] = NO_MOVEMENT;
		}
	}//if

	// check the southwest section for southwest movement
	if(unit_direction[SOUTHWEST_ROAD] == SOUTHWEST_MOVEMENT)
	{
		Node *p_southwest_node;
		if(is_vertical)
			p_southwest_node = p_node[Node::SOUTH];
		else
			p_southwest_node = p_node[Node::WEST];

		if(!p_southwest_node->addUnit(units[SOUTHWEST_ROAD], unit_owner[SOUTHWEST_ROAD]))
		{
			units[SOUTHWEST_ROAD] = Node::NONE;
			unit_owner[SOUTHWEST_ROAD] = NOT_OWNED;
			unit_direction[SOUTHWEST_ROAD] = NO_MOVEMENT;
		}
	}

	// move units which won't leave the road this step
	for(int i = NORTHEAST_ROAD; i < SOUTHWEST_ROAD+1; i++)
	{
		if(unit_direction[i] != NO_MOVEMENT)
		{
			int new_i = i + unit_direction[i];
			if(new_i < NORTHEAST_ROAD || new_i > SOUTHWEST_ROAD || units[new_i] != Node::NONE)
			{
				#ifdef DEBUG3
					lm.writeLog("Road::moveUnits: Not Moving a unit from %d to %d",
							i, new_i);
				#endif

				if(new_i < NORTHEAST_ROAD || new_i > SOUTHWEST_ROAD)
				{
				}else if(unit_owner[new_i] != unit_owner[i])
				{
					// have units fight
					if(Node::unitCombat(units[i], units[new_i]))
					{
						// have unit i replace unit new_i
						units[new_i] = units[i];
						unit_owner[new_i] = unit_owner[i];
						unit_direction[new_i] = unit_direction[i];
					}

					// remove unit i
					units[i] = Node::NONE;
					unit_owner[i] = NOT_OWNED;
					unit_direction[i] = NO_MOVEMENT;		
				}else if(unit_owner[new_i] == unit_owner[i] && unit_direction[new_i] != unit_direction[i])
				{
					Node::Unit temp_units = units[i];
					int temp_unit_owner = unit_owner[i];
					int temp_unit_direction = unit_direction[i];
					
					units[i] = units[new_i];
					unit_owner[i] = unit_owner[new_i];
					unit_direction[i] = unit_direction[new_i];
					
					units[new_i] = temp_units;
					unit_owner[new_i] = temp_unit_owner;
					unit_direction[new_i] = temp_unit_direction;
				}
			}else
			{	
				#ifdef DEBUG3
					lm.writeLog("Road::moveUnits: Moving a unit from %d to %d",
							i, new_i);
				#endif
				units[new_i] = units[i];
				unit_owner[new_i] = unit_owner[i];
				unit_direction[new_i] = unit_direction[i];
	
				units[i] = Node::NONE;
				unit_owner[i] = NOT_OWNED;
				unit_direction[i] = NO_MOVEMENT;

				if(new_i > i)
					i = new_i;
			}
		}
	}
}//moveUnits

///////////////////////////////////////////////////////////////////////////////////////
	
int Road::eventHandler(Event *e) {
	if(e->getType() == STEP_EVENT)
	{
		move_countdown--;
		if(move_countdown > 0)
			return 1;
		
		move_countdown = move_slowdown;
		moveUnits();
		return 1;
	}

	return 0;
}//eventHandler
  
/////////////////////////////////////////////////////////////////////////////////////////

void Road::setNode(Node *new_node, Node::Direction direction) {
	if(is_vertical && (direction == Node::EAST || direction == Node::WEST))
	{
		// do nothing	
	}else if(!is_vertical && (direction == Node::NORTH || direction == Node::SOUTH))
	{
		// do nothing
	}else
	{
		p_node[direction] = new_node;
	}
}//setNode

Node *Road::getNode(Node::Direction direction) {
	if(is_vertical && (direction == Node::EAST || direction == Node::WEST))
	{
		// do nothing	
	}else if(!is_vertical && (direction == Node::NORTH || direction == Node::SOUTH))
	{
		// do nothing
	}else
	{
		return p_node[direction];
	}
}//getNode
		
/////////////////////////////////////////////////////////////////////////////////////////
		
// argument: unit_type 			- the type of unit being added
//  		 movement_direction - the direction that the unit is moving,
//									should use the MOVEMENT constants
// adds a single unit to the road
// return: 0 if unit doesn't return, 1 if unit returns, and -1 if error
int Road::addUnit(Node::Unit new_unit_type, int movement_direction, int new_unit_owner) {
	if(movement_direction == NO_MOVEMENT)
	{
		#ifdef DEBUG3
			LogManager &lm = LogManager::getInstance();
			lm.writeLog("Road::addUnit: new Unit movement does not fit with this type of road");
		#endif
		return -1;
	}

	int road_section = NORTHEAST_ROAD;
	if(movement_direction == NORTHEAST_MOVEMENT)
	{
		road_section = SOUTHWEST_ROAD;
	}

	#ifdef DEBUG3
		LogManager &lm = LogManager::getInstance();
		lm.writeLog("Road::addUnit: Adding unit to %d road section, with movement %d",
				road_section, movement_direction);
	#endif

	if(units[road_section] != Node::NONE)
	{
		if(unit_owner[road_section] != new_unit_owner)
		{
			if(Node::unitCombat(units[road_section], new_unit_type))
			{
				// new unit dies
				return 0;
			}
			// otherwise have the unit move onto the road
		}else
			return 1;
	}

	// the unit moves onto the road (possibly through conquest)
	units[road_section] = new_unit_type;
	unit_direction[road_section] = movement_direction;
	unit_owner[road_section] = new_unit_owner;
	return 0;
}//addUnit

///////////////////////////////////////////////////////////////////////////////////////////

void Road::draw() {
#ifdef DEBUG3
	LogManager &lm = LogManager::getInstance();
	lm.writeLog("Node::draw: Drawing Node #%d, Sprite Color is %d", getId(),
			getSprite()->getColor());
#endif
	GameObject::draw();
	GraphicsManager &gm = GraphicsManager::getInstance();

	for(int i = 0; i < 3; i++)
	{
#ifdef DEBUG3
		LogManager &lm = LogManager::getInstance();
		lm.writeLog("Road::draw: Drawing unit #%d on Road %d, the unit is a %d",
				i, getId(), units[i]);
#endif

		int unit_color;
		switch(unit_owner[i])
		{
		case NOT_OWNED:
			unit_color = NOT_OWNED_COLOR;
			break;
		case PLAYER_OWNED:
			unit_color = PLAYER_OWNED_COLOR;
			break;
		default:
			unit_color = AI_OWNED_COLOR;
		};

		Position unit_pos = getPos();
		if(is_vertical)
			unit_pos.setY(unit_pos.getY()+i);
		else
			unit_pos.setX(unit_pos.getX()+(2-i)*2);

		char unit_ch;
		switch(units[i])
		{
			case Node::SWORDSMEN:
				unit_ch = SWORDSMEN_CHAR;
				break;
			case Node::HORSEMEN:
				unit_ch = HORSEMEN_CHAR;
				break;
			case Node::PIKEMEN:
				unit_ch = PIKEMEN_CHAR;
				break;
			default:
				if(is_vertical)
					unit_ch = VERT_CHAR;
				else
					unit_ch = HORI_CHAR;
		};
		
		gm.drawCh(unit_pos, unit_ch, unit_color);
	}//for	
}//draw
