/*
 * Written by Cordell Zebrose & Michael Metzler
 * Node.h
 *
 * Node is a GameObject which contains all the information for
 *	a single node. Each node is connected to up to 4 Roads, 
 *	up to 2 buildings, and up to 9 troops of a each type.
 */

#ifndef NODE_H
#define NODE_H
 
#include "GameObject.h"
#include "KeyboardEvent.h"
#include "StepEvent.h"

#define NOT_OWNED 0
#define PLAYER_OWNED 1
#define AI_OWNED 2

#define NOT_OWNED_COLOR WHITE_BLACK_PAIR
#define PLAYER_OWNED_COLOR BLUE_BLACK_PAIR
#define AI_OWNED_COLOR RED_BLACK_PAIR

#define NOT_OWNED_SELECTED YELLOW_BLACK_PAIR
#define PLAYER_OWNED_SELECTED GREEN_BLACK_PAIR
#define AI_OWNED_SELECTED MAGENTA_BLACK_PAIR

#define MAX_GARRISON 9

#define COMBAT_UNFAIR 7
#define COMBAT_FAIR 5

#define NUM_BUILDING_TYPES 4
#define NO_BUILDING 0
#define SWORDSMEN_BARRACKS 1
#define HORSEMEN_BARRACKS 2
#define PIKEMEN_BARRACKS 3
#define MINE 4

#define COST_NO_BUILDING 0
#define COST_S_BUILDING 150
#define COST_H_BUILDING 160
#define COST_P_BUILDING 190
#define COST_M_BUILDING 90

#define COST_SWORDSMEN 50
#define COST_HORSEMEN 40
#define COST_PIKEMEN 30

#define MINE_OUTPUT 40

#define CHAR_NO_BUILDING '_'
#define CHAR_S_BUILDING 'S'
#define CHAR_H_BUILDING 'H'
#define CHAR_P_BUILDING 'P'
#define CHAR_M_BUILDING 'M'

// required for mutual dependence
class Road;

class Node : public GameObject {
	public:
		enum Direction { NORTH, SOUTH, EAST, WEST };
		enum Unit { SWORDSMEN, HORSEMEN, PIKEMEN, NONE };

	 protected:
		// a pointer to the roads connecting to the node
		//	p_road uses enum Direction for indexing, for example p_road[NORTH]
		Road *p_road[4];	
							
		// a list of the four roads with information on unit movements.
		//	The value of these movements will represent the type of unit which
		// 	is moving from the node to that road.
		Unit road_movements[4];
							
		// a counter for the number of troops in the node
		// 	garrison uses enum Unit for indexing, for example garrison[HORSEMEN]
		int garrison[3];    
		
		int buildings[2];
		int cost[NUM_BUILDING_TYPES+1];
		
		// extra resources which have been mined
		int surplus_resources;
							
		// the current owner of this node:
		//	0 = no owner
		//	1 = Player
		//	2 = AI
		//  etc..
		int owner;

		bool is_selected;

		// required to switch Sprite colors during drawing
		int color;

		// move units for one step
		void step();	

		void switchColor(int new_color);	
		
	 public:
		// arguments: 	x - the x position on the node graph, must be converted to world coordinates
		//				y - the y position on the node graph, must be converted to world coordinates
		// creates an empty unowned node which isn't connected to any roads and has no units
		Node(int x, int y);
		~Node();
		
		int eventHandler(Event *e);

		int getSword();
		int getHorse();
		int getPike();
  
		void setRoad(Road *new_road, Direction direction);
		Road *getRoad(Direction direction);
		
		// argument: unit_type - the type of unit being added
		// adds a single unit to the node
		// return: 0 if unit doesn't return, 1 if unit returns, and -1 if error
		int addUnit(Unit unit_type, int new_unit_owner);
		
		// arguments: direction - the direction to move in
		//			  unit_type - the type of unit being added
		// Starts to move all units of type unit_type in the 
		// 	direction indicated until all units have moved.
		// return: 0 if successful, 1 if there are no units
		//			available, and -1 if the move failed
		int moveUnits(Unit unit_type, int direction);
		
		int getBuilding(int index);
		
		// returns the cost of the building
		int checkBuildingCost(int building_type);
		
		// Adds a building to the node according to building_type
		void addBuilding(int building_type);
		
		// Removes all the buildings on the node
		// returns the costs of those buildings
		int removeBuildings();
		
		void buildingStep();
		int getResources();
		
		void setOwner(int new_owner);
		int getOwner();
		
		void setIsSelected(bool new_is_selected);
		bool getIsSelected();
		
		// override the GameObject draw method, so that the color
		void draw();

		// fights combat between the given units and return the winner
		// return true if unit_a wins, and false if unit_b wins
		static bool unitCombat(Unit unit_a, Unit unit_b);
};

#endif //NODE_H
