/*
 * Written by Cordell Zebrose & Michael Metzler
 * Road.h
 *
 * Road is a GameObject which contains all the information for
 *	a single road section. Each road is either a vertical road
 *  or a horizontal road. A road section is connected to exactly
 *  2 nodes and contains up to 3 units.
 */

#ifndef ROAD_H
#define ROAD_H
 
#include "GameObject.h"
#include "Node.h"

// these should be used with unit_direction to determine which direction a given unit is moving in
#define NO_MOVEMENT 0
#define NORTHEAST_MOVEMENT -1
#define SOUTHWEST_MOVEMENT 1

// define road sections
#define NORTHEAST_ROAD 0
#define MIDDLE_ROAD 1
#define SOUTHWEST_ROAD 2

#define SWORDSMEN_CHAR 'S'
#define HORSEMEN_CHAR 'H'
#define PIKEMEN_CHAR 'P'
#define VERT_CHAR '|'
#define HORI_CHAR '-'

class Road : public GameObject {

	 protected:
		bool is_vertical;
	 
		// a pointer to the nodes connecting to the node
		//	p_node uses enum Direction for indexing, for example p_node[NORTH]
		// vertical roads only use Directions NORTH/SOUTH and horizontal roads
		//	only use Directions EAST/WEST. If the wrong direction is used, always
		//  return an error.
		Node *p_node[4];
							
		// a list of units which are located along the road
		//  the index shows where on the road the unit is,
		//			0 = northeast section of road
		//			1 = middle of road
		//			2 = southwest section of road
		Node::Unit units[3];
		
		// a list which is associated with units, it shows which direction each unit is moving in
		//	This array should only use the defined constants NO_MOVEMENT, NORTHEAST_MOVEMENT, and
		//	SOUTHWEST_MOVEMENT.
		int unit_direction[3]; 
		
		// indicates the owners of the unit per road section
		int unit_owner[3];
		
		int move_slowdown;
		int move_countdown;

		// move all the units on the road, if a unit moves off the road then send it to the connected
		//	node. NOTE: all three unit lists will be modified by this function
		void moveUnits();

		void initPosition(Node *p_northeast_node);

		void switchColor(int new_color);

	 public:
		// All roads must be connected to two nodes and be either vertical or horizontal
		Road(bool init_is_vertical, Node *init_northeast_node, Node *init_southwest_node);
		~Road();
		
		int eventHandler(Event *e);
  
		void setNode(Node *new_node, Node::Direction direction);
		Node *getNode(Node::Direction direction);
		
		// argument: unit_type 			- the type of unit being added
		//  		 movement_direction - the direction that the unit is moving,
		//									should use the MOVEMENT constants
		// adds a single unit to the road
		// return: 0 if unit doesn't return, 1 if unit returns, and -1 if error
		int addUnit(Node::Unit new_unit_type, int movement_direction, int new_unit_owner);

		void draw();
};

#endif //ROAD_H
