/*
  WorldManager.h
  Written by Cordell Zebrose
  
  WorldManager stores information about the game objects in the world, and
	sends a STEP_EVENT to them all every step.
*/

#ifndef _WORLD_MANAGER_H_
#define _WORLD_MANAGER_H_

#include "Manager.h"
#include "GameObjectList.h"
#include "Box.h"

#define MAX_ALTITUDE 2

class WorldManager : public Manager {
	private:
		// force singleton implementation
		WorldManager();
		WorldManager(const WorldManager &wm);
		void operator=(const WorldManager &wm);
		
	protected:
		// list of all game objects in the world
		GameObjectList obj_list;
		
		// list of all game objects to be deleted
		GameObjectList del;
		
	public:
		// returns the singleton instance of WorldManager
		static WorldManager &getInstance();
		
		// startup the WorldManager
		int startup();
		
		// shutdown the WorldManager
		void shutdown();
		
		// allow controlled interation with obj_list
		GameObjectList getAllObjects();
		int insertObj(GameObject *p_go);
		int removeObj(GameObject *p_go);
		GameObject* findObj(int id);
		
		// update world, send a step event to all interested objects
		void update();	

		// draw all the objects in the world
		void draw();
		
		// check to see if two positions intersect
		// returns true if they do
		bool positionIntersect(Position p1, Position p2);
		
		// check to see if two boxes intersect
		// returns true if they do
		bool boxIntersect(Box b1, Box b2);
		
		// Checks to see if the GameObject p_go can move to Position new_loc
		// return: NULL if the move doesn't cause a collision, otherwise
		//  	return a pointer to the GameObject which p_go is colliding
		//		with.
		GameObject *isCollision(GameObject *p_go, Position new_loc);
		
		// This is a request to move the GameObject p_go to Position new_loc.
		// 	If the move would cause a collision then don't move the object and
		//	return -1. Otherwise, move the object and return 0.
		int moveObj(GameObject *p_go, Position new_loc);
		
		// Indictate the GameObject p_go will be deleted at the end of the current
		//		game loop.
		// Returns 0 if successful, otherwise -1.
		int markForDelete(GameObject *p_go);
}; //WorldManager

#endif //_WORLD_MANAGER_H_
