/*
  Clock.cpp
  Written by Cordell Zebrose
*/

#include "Clock.h"
#include "LogManager.h"

///////////////////////////////////////////////

// start the timer in preperation for the first delta() call
Clock::Clock() {
#ifdef __CYGWIN__
	// if it is running on Cygwin
	QueryPerformanceCounter(&prev);
	QueryPerformanceFrequency(&frequency);
#else
	// if it is running on Linux
	clock_gettime(CLOCK_REALTIME, &prev_ts);
#endif	
}// Clock::Clock()

//////////////////////////////////////////////

Clock::~Clock() {

}// Clock::~Clock()

//////////////////////////////////////////////

// return the time (in milliseconds) since the last time this
// 		function was called, or -1 if there is an error
long int Clock::delta() {
	long int elapsed_time; // in microseconds
#ifdef __CYGWIN__
	// if running Cygwin
	LARGE_INTEGER curr;		// current tick count
	
	QueryPerformanceFrequency(&frequency); // check current CPU freq
	QueryPerformanceCounter(&curr);			// set curr
	
	elapsed_time = (long int)((curr.QuadPart - prev.QuadPart) * 1000000.0 /
						frequency.QuadPart);
	
	QueryPerformanceCounter(&prev);
#else
	// if running Linux
	struct timespec curr_ts;
	long int curr_microsec, prev_microsec;
	
	clock_gettime(CLOCK_REALTIME, &curr_ts); // stop timer
	
	curr_microsec = curr_ts.tv_sec*1000000 + curr_ts.tv_nsec/1000;
	prev_microsec = prev_ts.tv_sec*1000000 + prev_ts.tv_nsec/1000;
	elapsed_time = curr_microsec - prev_microsec;
	
	clock_gettime(CLOCK_REALTIME, &prev_ts); // restart the timer
#endif

	if(elapsed_time < 0)
	{
		LogManager &lm = LogManager::getInstance();
#ifdef DEBUG1
		lm.writeLog("Clock::delta(): Error - the elapsed time was %ld, returning -1", elapsed_time);
#endif
		return -1;
	}else
	{
		return elapsed_time / 1000;
	}//if-else
}// Clock::delta

///////////////////////////////////////////////////

// arguement: ms - number of milliseconds to wait
// Busy waits the program for ms milliseconds
void Clock::busySleep(int ms) {	
	long int elapsed_time = 0; // in milliseconds
#ifdef __CYGWIN__
	// if running Cygwin
	Sleep(ms);
	return;
#else
	// if running Linux
	struct timespec curr_ts;
	struct timespec start_ts;
	long int curr_microsec, start_microsec;
	
	clock_gettime(CLOCK_REALTIME, &start_ts); // start timer
	while(elapsed_time < ms)
	{
		clock_gettime(CLOCK_REALTIME, &curr_ts); // check timer
		
		curr_microsec = curr_ts.tv_sec*1000000 + curr_ts.tv_nsec/1000;
		start_microsec = start_ts.tv_sec*1000000 + start_ts.tv_nsec/1000;
		elapsed_time = (curr_microsec - start_microsec) / 1000;
	}// while
	return;
#endif
}// Clock::busySleep
