/*
  GameManager.cpp
  Written by Cordell Zebrose
*/

#include "GameManager.h"
#include "LogManager.h"
#include "WorldManager.h"
#include "GraphicsManager.h"
#include "InputManager.h"
#include "ResourceManager.h"
#include "Clock.h"

#include <cstdlib>

////////////////////////////////////////////////////////////////////////////

// Made private to prevent the user from making multiple GameManager objects
GameManager::GameManager() {
}// GameManager::GameManager

////////////////////////////////////////////////////////////////////////////

// Private to prevent copying
GameManager::GameManager(const GameManager &gm) {
}// GameManager::GameManager

//////////////////////////////////////////////////////////////////////////////

// Private to prevent copying
void GameManager::operator=(const GameManager &gm){
}// GameManager::operator=

///////////////////////////////////////////////////////////////////////////

GameManager::~GameManager() {
	if(is_started)
	{
		shutdown();
	}// if
}// GameManager::~GameManager

///////////////////////////////////////////////////////////////////////////

// returns the singleton instance of the GameManager
GameManager &GameManager::getInstance() {
	static GameManager gm;
	return gm;
}// GameManager::getInstance()

/////////////////////////////////////////////////////////////////////////////

// run the game loop with the default arguements
void GameManager::run() {
	run(FRAME_TIME);
}// run()

/////////////////////////////////////////////////////////////////////////////

// arguement: frame_time - number of milliseconds between frames
// run the game loop, 
//	use the arguement for the time between frames
void GameManager::run(int frame_time) {
	time_between_frames = frame_time; // number of milliseconds allocated for each frame
	Clock clock;
	long int elapsed_time = clock.delta(); // number of millisecond since the frame started processing
	LogManager &lm = LogManager::getInstance();
	int game_counter = 0;
	
	while(!game_over) {
#ifdef DEBUG2
		lm.writeLog("GameManager::run: **************************NEW**GAME**TICK*************************");
#endif
	
		// start clock
		clock.delta();
		
#ifdef DEBUG3
		// start Manager-by-Manager clock
		Clock manager_clock;
		manager_clock.delta();
#endif
	
		// Get input from keyboard/mouse
		InputManager &im = InputManager::getInstance();
		im.getInput();
		
#ifdef DEBUG3
		lm.writeLog("GameManager::run: InputManager.getInput took %ld milliseconds", manager_clock.delta());
#endif
		
		// Update world state
		WorldManager &wm = WorldManager::getInstance();
		wm.update();

#ifdef DEBUG3
		lm.writeLog("GameManager::run: WorldManager.update took %ld milliseconds", manager_clock.delta());
#endif
		
		// Draw new screen to back buffer
		wm.draw();
		
#ifdef DEBUG3
		lm.writeLog("GameManager::run: WorldManager.draw took %ld milliseconds", manager_clock.delta());
#endif
		
		// Swap back buff to current buffer
		GraphicsManager &graph_man = GraphicsManager::getInstance();
		graph_man.swapBuffers();
		
#ifdef DEBUG3
		lm.writeLog("GameManager::run: GraphicsManager.swapBuffers took %ld milliseconds", manager_clock.delta());
#endif
		
		// Measure how Long Last Loop took		
		elapsed_time = clock.delta();
		if(elapsed_time < time_between_frames)
		{
#ifdef DEBUG2
			lm.writeLog("GameManager::run: Game cycle took %ld milliseconds, so sleep for %f milliseconds",
				elapsed_time,
				((double)time_between_frames - (double)elapsed_time));
#endif
			// Sleep for (TARGET_TIME - elapsed)
			clock.busySleep(time_between_frames - elapsed_time);
		}else
		{
#ifdef DEBUG2
			lm.writeLog("GameManager::run: the frame took %ld milliseconds to process", 
																					elapsed_time);
#endif
		}// if-else
	}// while
}// GameManager::run()
		
/////////////////////////////////////////////////////////////////////////////

// arguments: append - true, if log file should append to the existing log file
//						false, if log file should overwrite the existing log file
//				flush - true, if buffer should be flushed after every log entry
//				seed - the random seed to use
// starts up all the Managers
// return: returns 0 on success, or -1 on error
int GameManager::startup() {
	return startup(true, true, time(NULL));
}// GameManager::startup

////////////////////////////////////////////////////////////////////////////////

int GameManager::startup(bool append, bool flush) {
	return startup(append, flush, time(NULL));
}// GameManager::startup

///////////////////////////////////////////////////////////////////////////////
int GameManager::startup(bool append, bool flush, time_t seed) {
	// start LogManager
	LogManager &lm = LogManager::getInstance();
	lm.startup(append, flush);
#ifdef DEBUG1
	lm.writeLog("GameManager::startup: GameManager starting up...");
#endif
	
	// start ResourceManager
	ResourceManager &rm = ResourceManager::getInstance();
	rm.startup();
	
	// start WorldManager
	WorldManager &wm = WorldManager::getInstance();
	wm.startup();
	
	// start GraphicsManager
	GraphicsManager &gm = GraphicsManager::getInstance();
	gm.startup();
	
	// start InputManager
	InputManager &im = InputManager::getInstance();
	im.startup();
	
	// seed rand()
	srand(seed);

	is_started = true;
	return 0;
}// GameManager::startup
		
/////////////////////////////////////////////////////////////////////////////////

// shuts down all the Managers
void GameManager::shutdown() {
	LogManager &lm = LogManager::getInstance();
#ifdef DEBUG1
	lm.writeLog("GameManager::shutdown: Managers shutting down");
#endif
	
	// shutdown InputManager
	InputManager &im = InputManager::getInstance();
	im.shutdown();
	
	// shutdown GraphicsManager
	GraphicsManager &gm = GraphicsManager::getInstance();
	gm.shutdown();
	
	// shutdown WorldManager
	WorldManager &wm = WorldManager::getInstance();
	wm.shutdown();
	
	// shutdown ResourceManager
	ResourceManager &rm = ResourceManager::getInstance();
	rm.shutdown();
	
#ifdef DEBUG1
	lm.writeLog("GameManager::shutdown: GameManager shutting down");
#endif
	
	// shutdown LogManager
	lm.shutdown();
	
	is_started = false;
}// GameManager::shutdown
	
////////////////////////////////////////////////////////////////////////////////
	
// set the value of game_over, game ends if true
void GameManager::setGameOver(bool g_o) {
	game_over = g_o;
}// setGameOver
