/*
  GameObject.cpp
  Written by Cordell Zebrose
*/

#include "GameObject.h"
#include "WorldManager.h"
#include "LogManager.h"
#include "GraphicsManager.h"

#include <math.h>

////////////////////////////////////////////////////

// create a default game object, with Position (0,0) and
//		type "GameObject"
GameObject::GameObject() {
	init(Position(), DEFAULT_TYPE);
}// GameObject()

//////////////////////////////////////////////////////

// create a game object at Position p and type "GameObject"
GameObject::GameObject(Position p) {
	init(p, DEFAULT_TYPE);
}// GameObject()

/////////////////////////////////////////////////////////

// create a game object at Position(0,0) and type t
GameObject::GameObject(std::string t) {
	init(Position(), t);
}// GameObject

/////////////////////////////////////////////////////////

// create a game object at Position p and type t
GameObject::GameObject(Position p, std::string t) {
	init(p, t);
}// GameObject

/////////////////////////////////////////////////////////

GameObject::~GameObject() {
	LogManager &lm = LogManager::getInstance();
#ifdef DEBUG2
	lm.writeLog("GameObject::~GameObject: Removing GameObject #%d from the world", id);
#endif
	WorldManager &wm = WorldManager::getInstance();
	if(wm.findObj(id) != NULL)
		wm.removeObj(this);
}// ~GameObject

//////////////////////////////////////////////////////////

void GameObject::init(Position p, std::string t) {
	static int nextId = 0;
	nextId++;
	
	id = nextId;
	pos = p;
	type = t;
	is_solid = true;
	altitude = 0;
	x_velocity_countdown = 0;
	y_velocity_countdown = 0;
	
	LogManager &lm = LogManager::getInstance();
#ifdef DEBUG2
	lm.writeLog("GameObject::init: Adding GameObject #%d to the world", id);
#endif
	
	WorldManager &wm = WorldManager::getInstance();
	wm.insertObj(this);
}// init
		
/////////////////////////////////////////////////////////

// set Position pos to new_pos
void GameObject::setPos(Position new_pos) {
	pos = new_pos;
	bounding_box.setCorner(new_pos);
}// setPos

/////////////////////////////////////////////////////////

// returns Position pos
Position GameObject::getPos() {
	return pos;
}// getPos
		
////////////////////////////////////////////////////////

// set type to new_type
void GameObject::setType(std::string new_type) {
	type = new_type;
}// setType

//////////////////////////////////////////////////////

// returns type
std::string GameObject::getType() {
	return type;
}// getType

//////////////////////////////////////////////////////////

// set id to new_id
void GameObject::setId(int new_id) {
	id = new_id;
}// setId

///////////////////////////////////////////////////////////

// returns id
int GameObject::getId() {
	return id;
}// getId

//////////////////////////////////////////////////////////

// set is_solid to new_solid
void GameObject::setSolid(bool new_solid) {
	is_solid = new_solid;
}// setSolid

///////////////////////////////////////////////////////////

// returns is_solid
bool GameObject::isSolid() {
	return is_solid;
}// getSolid

///////////////////////////////////////////////////////////

// set altitude to new_altitude
void GameObject::setAltitude(int new_altitude) {
	altitude = new_altitude;
}// setAltitude

//////////////////////////////////////////////////////////

// return altitude
int GameObject::getAltitude() {
	return altitude;
}// getAltitude

////////////////////////////////////////////////////////////

// set p_sprite to new_sprite
void GameObject::setSprite(Sprite *new_sprite) {
	p_sprite = new_sprite;
	
	Box new_bounding_box(pos, p_sprite->getWidth(), p_sprite->getHeight());
	setBoundingBox(new_bounding_box);
}// setSprite

////////////////////////////////////////////////////////

Sprite *GameObject::getSprite() {
	return p_sprite;
}// getSprite

////////////////////////////////////////////////////////////

void GameObject::setSpriteCentered(bool new_sprite_centered) {
	sprite_centered = new_sprite_centered;
}// setSpriteCentered

////////////////////////////////////////////////////////////

bool GameObject::isCentered() {
	return sprite_centered;
}// isCentered

///////////////////////////////////////////////////////////

void GameObject::setBoundingBox(Box new_bounding_box) {
	bounding_box = new_bounding_box;
}// setBoundingBox

////////////////////////////////////////////////////////////

// returns the Bounding Box, accounting for centering
Box GameObject::getBoundingBox() {
	bounding_box.setCorner(pos);
	
	// center the bounding box, if appropriate
	if(sprite_centered)
	{
		int offset_x = bounding_box.getHorizontal()/2;
		int offset_y = bounding_box.getVertical()/2;
		
		Position bounding_box_corner(bounding_box.getCorner().getX() - offset_x,
									 bounding_box.getCorner().getY() - offset_y);
		bounding_box.setCorner(bounding_box_corner);
	}// if
	
	return bounding_box;
}// getBoundingBox

////////////////////////////////////////////////////////////

// set x velocity
void GameObject::setXVelocity(float new_x_velocity) {
	x_velocity = new_x_velocity;
}// setXVelocity

////////////////////////////////////////////////////////////

// set y velocity
void GameObject::setYVelocity(float new_y_velocity) {
	y_velocity = new_y_velocity;
}// setYVelocity

////////////////////////////////////////////////////////////

// return x velocity
float GameObject::getXVelocity() {
	return x_velocity;
}// getXVelocity

/////////////////////////////////////////////////////////////

// return y velocity
float GameObject::getYVelocity() {
	return y_velocity;
}// getYVelocity

/////////////////////////////////////////////////////////////
		
// perform 1 step of horizontal velocity movement
// return the number of steps the object should move
int GameObject::getXVelocityStep() {
	// see if there is an x-velocity 
	if(!x_velocity)
		return 0;
		
	// see if time to move
	x_velocity_countdown--;
	if (x_velocity_countdown > 0)
		return 0;
		
	// ok, time to move, so figure out how far
	int step = 0;
	
	do {
		x_velocity_countdown += fabs(1/x_velocity);
		if(x_velocity < 0)
			step--;
		else
			step++;
	} while (x_velocity_countdown <= 0);
	
	return step;
}// getXVelocityStep

/////////////////////////////////////////////////////////////////

// perform 1 step of horizontal velocity movement
// return the number of steps the object should move
int GameObject::getYVelocityStep() {
	// see if there is an x-velocity 
	if(!y_velocity)
		return 0;
		
	// see if time to move
	y_velocity_countdown--;
	if (y_velocity_countdown > 0)
		return 0;
		
	// ok, time to move, so figure out how far
	int step = 0;
	
	do {
		y_velocity_countdown += fabs(1/y_velocity);
		if(y_velocity < 0)
			step--;
		else
			step++;
	} while (y_velocity_countdown <= 0);
	return step;
}// getYVelocityStep

////////////////////////////////////////////////////////////////

// if the default eventHandler() method is called, ignore the Event
int GameObject::eventHandler(Event *p_event) {
	return 0;
}// eventHandler

////////////////////////////////////////////////////////////

// draw this GameObject, base class doesn't do anything
void GameObject::draw() {
	// if the sprite isn't defined, then do nothing
	if(!p_sprite)
	{
		return;
	}//if

	// draw current frame
	GraphicsManager &gm = GraphicsManager::getInstance();
	gm.drawFrame(pos, p_sprite->getFrame(), sprite_centered, p_sprite->getColor());
	
	// increment frame
	p_sprite->setFrameIndex(p_sprite->getFrameIndex()+1);
}// draw
