/*
	GraphicsManager.cpp
	Written by Cordell Zebrose
*/

#include "GraphicsManager.h"
#include "LogManager.h"
#include "Frame.h"

///////////////////////////////////////////////////////////////

// private default constructor to have singleton instance
GraphicsManager::GraphicsManager() {
	is_started = false;
}// GraphicsManager

/////////////////////////////////////////////////////////////////

// prevent the GraphicsManager from being copied
GraphicsManager::GraphicsManager(const GraphicsManager &gm) {
}//GraphicsManager

////////////////////////////////////////////////////////////////

// prevent the GraphicsManager from being copied
void GraphicsManager::operator=(const GraphicsManager &gm) {
}//operator=

///////////////////////////////////////////////////////////////
		
GraphicsManager::~GraphicsManager() {
	if(is_started)
	{
		shutdown();
	}//if
}//~GraphicsManager()
		
////////////////////////////////////////////////////////////////
		
// returns the singleton instance of the GraphicsManager
GraphicsManager &GraphicsManager::getInstance() {
	static GraphicsManager gm;
	return gm;
}// getInstance
		
/////////////////////////////////////////////////////////////

// quick helper function for initColorPairs()
// returns the color which matches the index
int getColorFromIndex(int index) {
	switch(index) {
		case 1:
			return COLOR_BLACK;
		case 2:
			return COLOR_RED;
		case 3:
			return COLOR_GREEN;
		case 4:
			return COLOR_YELLOW;
		case 5:
			return COLOR_BLUE;
		case 6:
			return COLOR_MAGENTA;
		case 7:
			return COLOR_CYAN;
		case 8:
			return COLOR_WHITE;
		default:
			return COLOR_BLACK;
	};//switch
}// getColorFromIndex

///////////////////////////////////////////////////////////////

// quick helper function for startup()
// initializes all the color pairs		
void initColorPairs() {
	init_pair(BLACK_BLACK_PAIR, COLOR_BLACK, COLOR_BLACK);
	init_pair(RED_BLACK_PAIR, COLOR_RED, COLOR_BLACK);
	init_pair(GREEN_BLACK_PAIR, COLOR_GREEN, COLOR_BLACK);
	init_pair(YELLOW_BLACK_PAIR, COLOR_YELLOW, COLOR_BLACK);
	init_pair(BLUE_BLACK_PAIR, COLOR_BLUE, COLOR_BLACK);
	init_pair(MAGENTA_BLACK_PAIR, COLOR_MAGENTA, COLOR_BLACK);
	init_pair(CYAN_BLACK_PAIR, COLOR_CYAN, COLOR_BLACK);
	init_pair(WHITE_BLACK_PAIR, COLOR_WHITE, COLOR_BLACK);
}//initColorPairs
		
///////////////////////////////////////////////////////////////
		
// start up the GraphicsManager
// returns 0 if ok, and -1 on error
int GraphicsManager::startup() {
	LogManager &lm = LogManager::getInstance();
#ifdef DEBUG1
	lm.writeLog("GraphicsManager::startup: Graphics Managers starting up");
#endif

	// start curses
	initscr();
	
	// allocate space for win1 & win2
	win1 = newwin(0, 0, 0, 0);
	win2 = newwin(0, 0, 0, 0);
	curr_win = win1;
	
	// get the terminal size
	getmaxyx(curr_win, window_vert, window_horiz);
	
	// handle cursor
	if(leaveok(win1, TRUE))
	{
#ifdef DEBUG1
		lm.writeLog("GraphicsManager::startup: Error on leaveok(win1, TRUE)");
#endif
	}//if
		
	if(leaveok(win2, TRUE))
	{
#ifdef DEBUG1
		lm.writeLog("GraphicsManager::startup: Error on leaveok(win2, TRUE)");
#endif
	}//if
	
	// enable color
	if(has_colors()) 
	{
		if(start_color())
		{
#ifdef DEBUG1
			lm.writeLog("GraphicsManager::startup: Error on start_color()");
#endif
		}// if
		initColorPairs();
	}// if
	
	// enable bold
	if( wattron(win1, A_BOLD) || wattron(win2, A_BOLD) )
	{
#ifdef DEBUG1
		lm.writeLog("GraphicsManager::startup: Error on wattron(win, A_BOLD)");
#endif
	}// if
	
	// clear screen
	if( werase(win1) || werase(win2))
	{
#ifdef DEBUG1
		lm.writeLog("GraphicsManager::startup: Error on werase(win)");
#endif
	}//if
	
	is_started = true;
	return 0;
}//startup
		
///////////////////////////////////////////////////////////////
		
// shuts down the GraphicsManager
void GraphicsManager::shutdown() {
	LogManager &lm = LogManager::getInstance();
	if(endwin())
	{
#ifdef DEBUG1
		lm.writeLog("GraphicsManager::shutdown(): Error on endwin()");
#endif
	}//if
		
	is_started = false;
}// shutdown

/////////////////////////////////////////////////////////////
		
// switch to the other window buffer to display the
//		next frame
// return 0 if ok, and -1 on error
int GraphicsManager::swapBuffers() {
	LogManager &lm = LogManager::getInstance();
	if(wrefresh(curr_win))
	{
#ifdef DEBUG1
		lm.writeLog("GraphicsManager::swapBuffers(): Error on wrefresh(curr_win)");
#endif
	}//if
		
	if(curr_win == win1)
	{	
		if(werase(win2))
		{
#ifdef DEBUG1
			lm.writeLog("GraphicsManager::swapBuffers(): Error on werase(win2)");
#endif
		}// if
		curr_win = win2;
	}else
	{
		if(werase(win1))
		{
#ifdef DEBUG1		
			lm.writeLog("GraphicsManager::swapBuffers(): Error on werase(win1)");
#endif
		}// if
			
		curr_win = win1;
	}// if-else
}// swapBuffers
		
////////////////////////////////////////////////////////////////

// arguments: 	ch_pos - the position of the character
//				ch - the character to be drawn
//  			color_pair - the color pair which should be used for the draw
// draw a character at Position world_pos 
// NOTE: (0,0) is at top-left of screen with +y going down
// Optional color arguments
// return 0 if ok, and -1 on error
int GraphicsManager::drawCh(Position ch_pos, char ch) {
	return drawCh(ch_pos, ch, COLOR_DEFAULT);
}// drawCh

int GraphicsManager::drawCh(Position ch_pos, char ch, int color_pair) {
	LogManager &lm = LogManager::getInstance();
	int error = 0;
	
	// turn on color
	if(wattron(curr_win, COLOR_PAIR(color_pair)))
	{
#ifdef DEBUG2
		lm.writeLog("GraphicsManager::drawCh(): Error turning on color for \
						color_pair %d", color_pair);
#endif
		error -= 1;
	}// if
		
	
	// draw char
	if(mvwaddch(curr_win, ch_pos.getY(), ch_pos.getX(), ch))
	{
#ifdef DEBUG2
		lm.writeLog("GraphicsManager::drawCh(): Character %c at Position(%d,%d) is off the screen", 
					ch, ch_pos.getX(), ch_pos.getY());
#endif
		error -= 1;
	}// if
	
	// turn off color
	if(wattroff(curr_win, COLOR_PAIR(color_pair)))
	{
#ifdef DEBUG2
		lm.writeLog("GraphicsManager::drawCh(): Error turning off color for \
						color_pair %d", color_pair);
#endif
		error -= 1;
	}// if
	
	return error;
}// drawCh
		
//////////////////////////////////////////////////////////////////////////

// arguments: 	world_pos 	- the position to draw the Frame on the screen
//			 	frame 		- the frame to draw
//				centered 	- is the frame centered on the position or
//								is the position in the upper-left corner of the frame
//				color		- what color pair should be used
// draw a frame at Position world_pos
// return 0 if success, and -1 on error
int GraphicsManager::drawFrame(Position world_pos, Frame frame, bool centered, int color) {
	// check for empty frame
	if(frame.getString().size() <= 0)
	{
		return -1;
	}// if
	
	// determine offset of Frame
	int offset_y = 0;
	int offset_x = 0;
	if(centered)
	{
		offset_y = frame.getHeight() / 2;
		offset_x = frame.getWidth() / 2;
	}//if
	
	// get frame data
	string frame_data = frame.getString();
	for(int y = 0; y < frame.getHeight(); y++)
	{
		for(int x = 0; x < frame.getWidth(); x++)
		{
			Position curr_pos( 	world_pos.getX() - offset_x + x,
								world_pos.getY() - offset_y + y);
			drawCh(curr_pos, frame_data[y*frame.getWidth() + x], color);
		}//for
	}//for
}// drawFrame
		
/////////////////////////////////////////////////////////////////////
		
// return window_horiz or window_vert
int GraphicsManager::getHorizontal() {
	return window_horiz;
}// getHorizontal

int GraphicsManager::getVertical() {
	return window_vert;
}// getVertical
