/*
	InputManager.cpp
	Written by Cordell Zebrose
*/

#include "InputManager.h"
#include "LogManager.h"
#include "GraphicsManager.h"
#include "KeyboardEvent.h"
#include "Clock.h"
#include "MouseEvent.h"

///////////////////////////////////////////////////////

// singleton implementation
InputManager::InputManager() {
}//InputManager

InputManager::InputManager(const InputManager &im) {
}// InputManager

void InputManager::operator=(const InputManager &im) {
}// operator=
	
/////////////////////////////////////////////////////////

InputManager::~InputManager() {
	if(is_started)
		shutdown();
}// ~InputManager
		
///////////////////////////////////////////////////////
		
// return the only instance of InputManager
InputManager &InputManager::getInstance(){
	static InputManager im;
	return im;
}// getInstance
		
/////////////////////////////////////////////////////////
		
// only allow KeyboardEvent and MouseEvent
bool InputManager::isValid(string event_name) {
	return 	event_name.compare(KEYBOARD_EVENT) == 0 || 
			  event_name.compare(MOUSE_CLICK_EVENT) == 0;
}// isValid
		
/////////////////////////////////////////////////////////
		
// start up the InputManager
// return 0 on started, otherwise -1
int InputManager::startup() {
	LogManager &lm = LogManager::getInstance();
#ifdef DEBUG1
	lm.writeLog("InputManager::startup: InputManager starting up");
#endif
	int error = 0;

	// check to make sure that the GraphicsManager has started
	GraphicsManager &graph_man = GraphicsManager::getInstance();
	if(!graph_man.isStarted())
	{
#ifdef DEBUG1
		lm.writeLog("InputManager::startup(): GraphicsManager hasn't started yet.");
#endif
		return -1;
	}// if
	
	// enable keypad
	if(keypad(stdscr, TRUE))
	{
#ifdef DEBUG1
		lm.writeLog("InputManager::startup: Error - Enabling keypad failed");
#endif
		error -= 1;
	}// if
	
	// Disable line buffering
	if(cbreak())
	{
#ifdef DEBUG1
		lm.writeLog("InputManager::startup: Error - Disabling line buffer failed");
#endif
		error -= 1;
	}// if
	
	// Turn off newline on output
	if(nonl())
	{
#ifdef DEBUG1
		lm.writeLog("InputManager::startup: Error - Turning off newline on output failed");
#endif
		error -= 1;
	}// if
	
	// Disable character echo
	if(noecho())
	{
#ifdef DEBUG1
		lm.writeLog("InputManager::startup: Error - Disabling character echo failed");
#endif
		error -= 1;
	}// if
	
	// Turn off cursor
	if(curs_set(0))
	{
#ifdef DEBUG1
		lm.writeLog("InputManager::startup: Error - turning off the cursor failed");
#endif
		error -= 1;
	}// if
	
	// Set nodelay
	if(nodelay(stdscr, TRUE))
	{
#ifdef DEBUG1
		lm.writeLog("InputManager::startup: Error - setting nodelay failed");
#endif
		error -= 1;
	}// if
	
	// Enable mouse events
	mmask_t mask = BUTTON1_CLICKED | BUTTON2_CLICKED |
				BUTTON1_DOUBLE_CLICKED | BUTTON2_DOUBLE_CLICKED;
	mousemask(mask, NULL);
	
	// Set is_started
	is_started = true;
	return 0;
}// startup
		
//////////////////////////////////////////////////////////////////////////////
		
// shutdown the InputManager
void InputManager::shutdown() {
	LogManager &lm = LogManager::getInstance();
#ifdef DEBUG1
	lm.writeLog("InputManager::shutdown: InputManager shutting down");
#endif
	
	// turn on the cursor
	if(curs_set(1))
	{
#ifdef DEBUG1
		lm.writeLog("InputManager::shutdown: Error - Turning on the cursor failed");
#endif
	}//if
	
	// set is_started
	is_started = false;
}// shutdown
		
///////////////////////////////////////////////////////////////////////////////
		
// retrieve the input from the keyboard and
// 	mouse.
void InputManager::getInput() {
	LogManager &lm = LogManager::getInstance();
	
#ifdef DEBUG2
	Clock input_timer;
	input_timer.delta();
#endif
	
	// get character
	int ch = getch();
	
	// check that there is input
	if(ch == ERR)
	{
		// no user input
		return;
	}
	
#ifdef DEBUG2
	lm.writeLog("InputManager::getInput: Grabbing the input character took %ld milliseconds", input_timer.delta());
#endif
	
	// check if mouse event
	MEVENT m_event;
	if( (ch == KEY_MOUSE) && (getmouse(&m_event) == OK) )
	{	
		MouseEvent mouse_event;
		mouse_event.setMouseX(m_event.x);
		mouse_event.setMouseY(m_event.y);
	
#ifdef DEBUG2
		lm.writeLog("InputManager::getInput: MouseEvent triggered at location %d,%d",
						mouse_event.getMouseX(), mouse_event.getMouseY());
#endif
	
#ifdef DEBUG2
		lm.writeLog("InputManager::getInput: Setting up MouseEvent position took %ld milliseconds", input_timer.delta());
#endif
		
#ifdef DEBUG2
		Clock c;
		c.delta();
#endif
		
		if(m_event.bstate & BUTTON1_CLICKED)
		{
			mouse_event.setMouseAction(LEFT_BUTTON_CLICK);
		}else if(m_event.bstate & BUTTON2_CLICKED)
		{
			mouse_event.setMouseAction(RIGHT_BUTTON_CLICK);
		}else if(m_event.bstate & BUTTON1_DOUBLE_CLICKED)
		{
			mouse_event.setMouseAction(LEFT_BUTTON_DOUBLECLICK);
		}else if(m_event.bstate & BUTTON2_DOUBLE_CLICKED)
		{
			mouse_event.setMouseAction(RIGHT_BUTTON_DOUBLECLICK);
		}else
		{
			mouse_event.setMouseAction(UNDEFINED);
		
#ifdef DEBUG2
			lm.writeLog("InputManager::getInput: Mouse Event is undefined and ignored");
#endif
			// ignore the event
			return;
		}//if-elseif-...-else
		
#ifdef DEBUG2
		lm.writeLog("InputManager::getInput: Setting up MouseEvent Action took %ld milliseconds", input_timer.delta());
#endif
		
		// send MouseEvent
		onEvent(&mouse_event);
		
#ifdef DEBUG2
		lm.writeLog("InputManager::getInput: Send out the Mouse Event took %ld milliseconds", input_timer.delta());
#endif
	}else
	{			
		// else send keyboard event
		KeyboardEvent keyboard_event;
		keyboard_event.setKey(ch);
		
#ifdef DEBUG2
		lm.writeLog("InputManager::getInput: KeyboardEvent triggered on Key '%c'", keyboard_event.getKey());
#endif
		
		onEvent(&keyboard_event);
	}// if-else
}// getInput
