/*
	ResourceManager.cpp
	Written by Cordell Zebrose
*/

#include "ResourceManager.h"
#include "LogManager.h"

#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>

///////////////////////////////////////////////////////

// singleton implementation
ResourceManager::ResourceManager() {
}//ResourceManager

ResourceManager::ResourceManager(const ResourceManager &rm) {
}// ResourceManager

void ResourceManager::operator=(const ResourceManager &rm) {
}// operator=
	
/////////////////////////////////////////////////////////

ResourceManager::~ResourceManager() {
	if(is_started)
		shutdown();
}// ~InputManager
		
///////////////////////////////////////////////////////
		
// return the only instance of ResourceManager
ResourceManager &ResourceManager::getInstance(){
	static ResourceManager rm;
	return rm;
}// getInstance
		
/////////////////////////////////////////////////////////
		
// start up the ResourceManager
// returns 0 if ok, and -1 on error
int ResourceManager::startup() {
#ifdef DEBUG1
	LogManager &lm = LogManager::getInstance();
	lm.writeLog("ResourceManager::startup: Resource Manager starting up");
#endif

	sprite_count = 0;

	is_started = true;
	return 0;
}// startup

////////////////////////////////////////////////////////

// shutdown the ResourceManager
void ResourceManager::shutdown() {
#ifdef DEBUG1
	LogManager &lm = LogManager::getInstance();
	lm.writeLog("ResourceManager::shutdown: Resource Manager shutting down");
#endif

	for(int i = 0; i < sprite_count; i++)
	{
		delete sprite[i];
	}// for
	
	is_started = false;
}// shutdown

////////////////////////////////////////////////////////

// helper function for loadSprite
// Removes a '\r' character from the argument string
void discardCR(string &str) {
	if(str[str.size()-1] == '\r')
	{
		str.erase(str.size()-1);
	}// if
}// discardCR

//---------------------LOADSPRITE-------------------------------

// helper function for loadSprite
// Parses the Sprite header lines
int readLineInt(ifstream &p_file, int *p_line_number, string tag) {
	string line ="";
	getline(p_file, line);
	discardCR(line);
	
	int loc_of_break = line.find(' ');
	string token(line.substr(0,loc_of_break));
	
	if(token.compare(tag) != 0)
	{
#ifdef DEBUG2
		LogManager &lm = LogManager::getInstance();
		lm.writeLog("ResourceManager::readLineInt(): Error - unexpected tag on line #%d", *p_line_number);
#endif
		return -1;
	}else
	{
		(*p_line_number)++;
		token = line.substr(loc_of_break+1);
		return atoi(token.c_str());
	}// if-else
}// readLineInt

//------------------LOADSPRITE----------------------------------------

// helper function for loadSprite
// Reads in the next frame of a Sprite
Frame readFrame(ifstream &p_file, int *p_line_number, int width, int height)
{
	string line;
	string frame_str = "";
	for(int i = 0; i < height; i++)
	{
		std::getline(p_file, line);
		discardCR(line);
		
		if(line.size() > width)
		{
#ifdef DEBUG1
			LogManager &lm = LogManager::getInstance();
			lm.writeLog("ResourceManager::readFrame: Error on line #%d - the line had too many characters",
					*p_line_number);
#endif

			Frame empty_frame;
			return empty_frame;		
		}// if
		
		frame_str += line;
		(*p_line_number)++;
	}// for
	
	std::getline(p_file, line);
	discardCR(line);
	if(line.compare("end") != 0)
	{
#ifdef DEBUG1
		LogManager &lm = LogManager::getInstance();
		lm.writeLog("ResourceManager::readFrame: Error on line #%d - expected 'end' keyword");
		
		Frame empty_frame;
		return empty_frame;
#endif
	}// if
	
	Frame new_frame(width, height, frame_str);
	return new_frame;
}// readFrame

//-------------------------------LOADSPRITE-------------------------------------------------

// argument: 	filename - the file to open
//				label 	 - the label for the sprite
// Loads the Sprite from the file and gives it
//	a label.
// returns 0 if successful, or -1 on error
int ResourceManager::loadSprite(string filename, string label) {
	string line;
	ifstream sprite_file (filename.c_str());
	Sprite *new_sprite = new Sprite(MAX_FRAMES_PER_SPRITE);
	new_sprite->setLabel(label);
	if(sprite_file.is_open())
	{
		int line_number = 1;
		int num_frames = 999;
		int curr_frames = 0;
		int frame_height;
		int frame_width;
		
		while (sprite_file.good() && new_sprite->getFrameCount() < num_frames)
		{
			string tag = "";
			// read a new line
			switch(line_number)
			{
				case 1:	
					tag = "frames";
					num_frames = readLineInt(sprite_file, &line_number, tag);
					
					// check for error
					if(num_frames < 0)
					{
#ifdef DEBUG1
						LogManager &lm = LogManager::getInstance();
						lm.writeLog("ResourceManager::loadSprite: Error - there was a problem reading in the number of frames on line #%d", 
										line_number);
#endif
						return -1;
					}// if
					break;
				case 2:
					frame_width = readLineInt(sprite_file, &line_number, "width");
					new_sprite->setWidth(frame_width);
					
					// check for error
					if(frame_width < 0)
					{
#ifdef DEBUG1
						LogManager &lm = LogManager::getInstance();
						lm.writeLog("ResourceManager::loadSprite: Error - there was a problem reading in the wdith of frames \
on line #%d", line_number);
#endif
						return -1;
					}// if
					break;
				case 3:
					frame_height = readLineInt(sprite_file, &line_number, "height");
					new_sprite->setHeight(frame_height);
					
					// check for error
					if(frame_height < 0)
					{
#ifdef DEBUG1
						LogManager &lm = LogManager::getInstance();
						lm.writeLog("ResourceManager::loadSprite: Error - there was a problem reading in the height of frames \
on line #%d", line_number);
#endif
						return -1;
					}// if
					break;
				default:		
#ifdef DEBUG3
					LogManager &lm = LogManager::getInstance();
					lm.writeLog("ResourceManager::loadSprite: %s sprite - Reading frame #%d on line #%d", 
								new_sprite->getLabel().c_str(), new_sprite->getFrameCount(), line_number);
#endif
				
					new_sprite->addFrame(
						readFrame(sprite_file, &line_number, frame_width, frame_height));
			};//switch
		}// while
		sprite_file.close();
	}else
	{
		LogManager &lm = LogManager::getInstance();
		lm.writeLog("ResourceManager::loadSprite: Failed to open the file");
		
		return -1;
	}// if-else
	
	// successfully loaded sprite
	sprite[sprite_count] = new_sprite;
	sprite_count++;
	return 0;
}// loadSprite
		
////////////////////////////////////////////////////////////////////////////////////
		
// argument: label - the label on the requested Sprite
// returns the Sprite with the label given
Sprite *ResourceManager::getSprite(string label) {
	for(int i = 0; i < sprite_count; i++)
	{
		if(label.compare(sprite[i]->getLabel()) == 0)
		{
			return sprite[i];
		}//if
	}//for
	
#ifdef DEBUG3
	LogManager &lm = LogManager::getInstance();
	lm.writeLog("ResourceManager::getSprite: Couldn't find %s sprite", label.c_str());
#endif
	
	return NULL;
}// getSprite

//////////////////////////////////////////////////////////////////////////////////

// show the sprite_count
int ResourceManager::getSpriteCount() {
	return sprite_count;
}// getSpriteCount
