/*
  WorldManager.cpp
  Written by Cordell Zebrose
*/

#include "WorldManager.h"
#include "LogManager.h"
#include "GraphicsManager.h"
#include "GameObjectList.h"
#include "GameObjectListIterator.h"
#include "StepEvent.h"
#include "OutEvent.h"
#include "CollisionEvent.h"

////////////////////////////////////////////////////////////////////

// default constructor does nothing
WorldManager::WorldManager() {
}//WorldManager

/////////////////////////////////////////////////////////////////////

// copy constructor does nothing		
WorldManager::WorldManager(const WorldManager &wm) {
}// WorldManager

////////////////////////////////////////////////////////////////////

// operator= does nothing
void WorldManager::operator=(const WorldManager &wm) {
}//operator=
		
/////////////////////////////////////////////////////////////////////

// returns the singleton instance of WorldManager
WorldManager &WorldManager::getInstance() {
	static WorldManager wm;
	return wm;
}//getInstance

////////////////////////////////////////////////////////////////////
		
// startup the WorldManager
// Creates the empty GameObjectList
int WorldManager::startup() {
	LogManager &lm = LogManager::getInstance();
#ifdef DEBUG1
	lm.writeLog("WorldManager::startup: WorldManager starting up");
#endif

	obj_list.clear();
	del.clear();
	
	is_started = true;
}// startup
		
/////////////////////////////////////////////////////////////////////

// shutdown the WorldManager
// 		deletes all the objects in the world
void WorldManager::shutdown() {
	is_started = false;

	LogManager &lm = LogManager::getInstance();
#ifdef DEBUG1
	lm.writeLog("WorldManager::shutdown: WorldManager shutting down");
#endif
	GameObjectListIterator iter = obj_list.createIterator();
	while(!iter.isDone())
	{
		if(removeObj(iter.currentObj()))
		{
#ifdef DEBUG1
			lm.writeLog("WorldManager::shutdown: Failed to remove an object from the world, moving on");
#endif
			iter.next();
		}// if
	}// while
}// shutdown
		
///////////////////////////////////////////////////////////////////////////

// returns the enter list of GameObjects in the world
GameObjectList WorldManager::getAllObjects() {
	return obj_list;
}// getAllObjects

//////////////////////////////////////////////////////////////////////////

// argument: p_go - pointer to the GameObject which is being added to the world
// Adds p_go to the world
// returns 0 on success, or -1 on error/failure
int WorldManager::insertObj(GameObject *p_go) {
	LogManager &lm = LogManager::getInstance();
	
	if(!is_started)
	{
#ifdef DEBUG3
		lm.writeLog("WorldManager::insertObj: Will not add %s #%d when the WorldManager is not running",
					p_go->getType().c_str(), p_go->getId());
#endif
		return -1;
	}//if

	if(p_go == NULL)
	{
#ifdef DEBUG1
		LogManager &lm = LogManager::getInstance();
		lm.writeLog("WorldManager::insertObj: Error - argument is a NULL pointer");
#endif
		return -1;
	}else
	{
		return obj_list.insert(p_go);
	}// if-else
}// insertObj()

///////////////////////////////////////////////////////////////////////////

// argument: p_go - pointer to the GameObject which is being removed from the world
// Removes p_go from the world, WARNING: the removed object is deleted from memory
// Returns 0 on success, or -1 on error/failure
int WorldManager::removeObj(GameObject *p_go) {
	LogManager &lm = LogManager::getInstance();
	
	if(p_go == NULL)
	{
#ifdef DEBUG1
		lm.writeLog("WorldManager::removeObj(): Error - argument is a NULL pointer");
#endif
		return -1;
	}else
	{
		if(obj_list.remove(p_go))
		{
#ifdef DEBUG2
			lm.writeLog("WorldManager::removeObj(): Couldn't remove object #%d from the world list",
							p_go->getId());
#endif
			return -1;
		}else
		{
#ifdef DEBUG2
			lm.writeLog("WorldManager::removeObj: Permenantly Deleting %s %d", 
							p_go->getType().c_str(), p_go->getId());
#endif
			delete p_go;
			return 0;
		}//if-else			
	}// if-else
}// removeObj

//////////////////////////////////////////////////////////////////////////////

// argument: id - the id of the GameObject that is being searched for
// returns 
GameObject* WorldManager::findObj(int id) {
	GameObject *ret_go = obj_list.find(id);
	return ret_go;
}// findObj()

/////////////////////////////////////////////////////////////////////////////
		
// update world, send a step event to all interested objects
void WorldManager::update() {
	LogManager &lm = LogManager::getInstance();
#ifdef DEBUG2
	lm.writeLog("WorldManager::update: Updating world, there are %d GameObjects to update", obj_list.getCount());
#endif

	// send out "step" event to all interested objects
	StepEvent s_e;
	onEvent(&s_e);
	
/*
	// move objects with velocity
	GameObjectListIterator vi = obj_list.createIterator();
	
	while(!vi.isDone())
	{
		// get the current object
		GameObject *p_go = vi.currentObj();
		
		// check that it worked
		if(!p_go)
		{
#ifdef DEBUG1
			LogManager &lm = LogManager::getInstance();
			lm.writeLog("WorldManager::update: Error - got a NULL pointer back from the list iterator");
#endif

			vi.next();
			break;
		}// if
		
		int x = p_go->getXVelocityStep();  // see how far moved x
		int y = p_go->getYVelocityStep();  // see how far moved y
		
		if(x != 0 || y != 0)
		{
			Position new_pos(p_go->getPos().getX() + x, 
							 p_go->getPos().getY() + y);
							 
			// move the object
			moveObj(p_go, new_pos);
		}// if
		
		// iterate vi
		vi.next();
	}// while
*/		
	// delete all marked objects
	GameObjectListIterator iter = del.createIterator();	
	while(!iter.isDone())
	{
		removeObj(iter.currentObj());		
		iter.next();
	}//while
	
#ifdef DEBUG3
	lm.writeLog("WorldManager::update: The marked GameObjects have all been deleted, clearing the delete list");
#endif
	
	del.clear();
}// update

//////////////////////////////////////////////////////////////////////////////////

// draw all the objects in the world
void WorldManager::draw() {
	LogManager &lm = LogManager::getInstance();
#ifdef DEBUG2
	lm.writeLog("WorldManager::draw: drawing all the objects in the world.");
#endif
	
	for(int alt = -MAX_ALTITUDE; alt <= MAX_ALTITUDE; alt++)
	{
		GameObjectListIterator iter = obj_list.createIterator();
		while(!iter.isDone())
		{
			GameObject *curr_obj = iter.currentObj();
			
			if(curr_obj->getAltitude() == alt)
			{
				if(curr_obj != NULL)
				{
					curr_obj->draw();
				}else
				{
#ifdef DEBUG1
					lm.writeLog("WorldManager::draw: Error - Found NULL pointer in the list of objects");
#endif
				}// if-else
			}//if
			iter.next();
		}// while
	}// for
}// draw

/////////////////////////////////////////////////////////////////////////////////////

// check to see if two positions intersect
// returns true if they do
bool WorldManager::positionIntersect(Position p1, Position p2) {
	return p1.getX() == p2.getX() && p1.getY() == p2.getY();
}// positionIntersect
		
////////////////////////////////////////////////////////////////////////////////////

// check to see if two boxes intersect
// returns true if they do
bool WorldManager::boxIntersect(Box b1, Box b2) {
	int x1_high = b1.getCorner().getX() + b1.getHorizontal();
	int x1_low = b1.getCorner().getX();
	int y1_high = b1.getCorner().getY() + b1.getVertical();
	int y1_low = b1.getCorner().getY();
	
	int x2_high = b2.getCorner().getX() + b2.getHorizontal();
	int x2_low = b2.getCorner().getX();
	int y2_high = b2.getCorner().getY() + b2.getVertical();
	int y2_low = b2.getCorner().getY();
	
	// check that x1 is higher than x2
	if(x1_low >= x2_high)
	{
		return false;
	// check that x1 is lower than x2
	}else if(x1_high <= x2_low)
	{
		return false;
	// check that y1 is higher than y2
	}else if(y1_low >= y2_high)
	{
		return false;
	// check that y1 is lower than y2
	}else if(y1_high <= y2_low)
	{
		return false;
	}else
	{
#ifdef DEBUG3
		LogManager &lm = LogManager::getInstance();
		lm.writeLog("WorldManager::boxIntersect(): There is an intersection between b1 (%d,%d);(%d,%d) and b2 (%d,%d);(%d,%d)",
					x1_low, y1_low, x1_high, y1_high, x2_low, y2_low, x2_high, y2_high);
#endif
		return true;
	}// if-elseif-else
}// boxIntersect
		
//////////////////////////////////////////////////////////////////////////////////////	
	
// Checks to see if the GameObject p_go can move to Position new_loc
// return: NULL if the move doesn't cause a collision, otherwise
//  	return a pointer to the GameObject which p_go is colliding
//		with.
GameObject *WorldManager::isCollision(GameObject *p_go, Position new_loc) {
	if(!p_go->isSolid())
	{
		// non-solid objects never collide
		return NULL;
	}// if
	
	GameObjectListIterator iter = obj_list.createIterator();
	while(!iter.isDone())
	{
		GameObject *p_temp_go = iter.currentObj();
		
		if(p_temp_go != p_go && p_temp_go->isSolid())
		{		
			if(boxIntersect(p_temp_go->getBoundingBox(), p_go->getBoundingBox()))
			{
#ifdef DEBUG3
				LogManager &lm = LogManager::getInstance();
				lm.writeLog("WorldManager::isCollision: There was a collision between %s #%d & %s #%d at Position %d,%d",
								p_go->getType().c_str(), p_go->getId(),
								p_temp_go->getType().c_str(), p_temp_go->getId(),
								new_loc.getX(), new_loc.getY());
#endif
				return p_temp_go;
			}//if
		}//if
		iter.next();
	}//while
	
	// no collision
	return NULL;
}// isCollision
		
//////////////////////////////////////////////////////////////////////////////////////////

// helper function for moveObj
// returns true if p is inside the world boundaries, otherwise false
bool insideWorld(Position p) {
	int x = p.getX();
	int y = p.getY();
	GraphicsManager &gm = GraphicsManager::getInstance();
	bool ret_val = x > 0 && y > 0 && x < gm.getHorizontal() && y < gm.getVertical();

	return ret_val;
}// insideWorld

// This is a request to move the GameObject p_go to Position new_loc.
// 	If the move would cause a collision then don't move the object and
//	return -1. Otherwise, move the object and return 0.
int WorldManager::moveObj(GameObject *p_go, Position new_loc) {	
	GameObject *p_temp_go = isCollision(p_go, new_loc);
	if(p_go->isSolid() && p_temp_go != NULL)
	{
		CollisionEvent c(p_go, p_temp_go, new_loc);
		p_go->eventHandler(&c);
		p_temp_go->eventHandler(&c);
		return -1;
	}else
	{
		bool was_inside = insideWorld(p_go->getPos());
		bool is_inside = insideWorld(new_loc);
				
		p_go->setPos(new_loc);
		
		// if the object just moved outside the world
		if(was_inside && !is_inside)
		{
			OutEvent ov;
			p_go->eventHandler(&ov);
		}// if
		return 0;
	}// if-else
}// moveObject

///////////////////////////////////////////////////////////////////////////////////////////

// Indictate the GameObject p_go will be deleted at the end of the current
//		game loop.
// Returns 0 if successful, otherwise -1.
int WorldManager::markForDelete(GameObject *p_go) {
	GameObjectListIterator iter = del.createIterator();
	while(!iter.isDone())
	{
		GameObject *curr_obj = iter.currentObj();
		if(curr_obj == p_go)
		{
			// object is already going to be deleted
			return 0;
		}//if
		iter.next();
	}//while
	
	// object isn't in the list
	return del.insert(p_go);
}// markForDelete

/////////////////////////////////////////////////////////////////////////////////////////////
